@@ -398,14 +398,15 @@ lua_link_init(char *libname, int verbose)
     }
     return OK;
 }
+#endif /* DYNAMIC_LUA */
 
+#if defined(DYNAMIC_LUA) || defined(PROTO)
     int
 lua_enabled(int verbose)
 {
     return lua_link_init((char *)p_luadll, verbose) == OK;
 }
-
-#endif /* DYNAMIC_LUA */
+#endif
 
 #if LUA_VERSION_NUM > 501
     static int
@@ -1177,10 +1178,8 @@ luaV_window_index(lua_State *L)
 	lua_pushinteger(L, w->w_cursor.lnum);
     else if (strncmp(s, "col", 3) == 0)
 	lua_pushinteger(L, w->w_cursor.col + 1);
-#ifdef FEAT_WINDOWS
     else if (strncmp(s, "width", 5) == 0)
-	lua_pushinteger(L, W_WIDTH(w));
-#endif
+	lua_pushinteger(L, w->w_width);
     else if (strncmp(s, "height", 6) == 0)
 	lua_pushinteger(L, w->w_height);
     /* methods */
@@ -1220,7 +1219,6 @@ luaV_window_newindex (lua_State *L)
 	w->w_cursor.col = v - 1;
 	update_screen(VALID);
     }
-#ifdef FEAT_WINDOWS
     else if (strncmp(s, "width", 5) == 0)
     {
 	win_T *win = curwin;
@@ -1231,7 +1229,6 @@ luaV_window_newindex (lua_State *L)
 	win_setwidth(v);
 	curwin = win;
     }
-#endif
     else if (strncmp(s, "height", 6) == 0)
     {
 	win_T *win = curwin;
@@ -1402,13 +1399,13 @@ luaV_buffer(lua_State *L)
 	if (lua_isnumber(L, 1)) /* by number? */
 	{
 	    int n = lua_tointeger(L, 1);
-	    for (buf = firstbuf; buf != NULL; buf = buf->b_next)
+	    FOR_ALL_BUFFERS(buf)
 		if (buf->b_fnum == n) break;
 	}
 	else { /* by name */
 	    size_t l;
 	    const char *s = lua_tolstring(L, 1, &l);
-	    for (buf = firstbuf; buf != NULL; buf = buf->b_next)
+	    FOR_ALL_BUFFERS(buf)
 	    {
 		if (buf->b_ffname == NULL || buf->b_sfname == NULL)
 		{
@@ -1715,6 +1712,8 @@ ex_luado(exarg_T *eap)
     const char *s = (const char *) eap->arg;
     luaL_Buffer b;
     size_t len;
+    buf_T *was_curbuf = curbuf;
+
     if (lua_init() == FAIL) return;
     if (u_save(eap->line1 - 1, eap->line2 + 1) == FAIL)
     {
@@ -1738,6 +1737,10 @@ ex_luado(exarg_T *eap)
     lua_replace(L, -2); /* function -> body */
     for (l = eap->line1; l <= eap->line2; l++)
     {
+	/* Check the line number, the command my have deleted lines. */
+	if (l > curbuf->b_ml.ml_line_count)
+	    break;
+
 	lua_pushvalue(L, -1); /* function */
 	luaV_pushline(L, curbuf, l); /* current line as arg */
 	lua_pushinteger(L, l); /* current line number as arg */
@@ -1746,6 +1749,9 @@ ex_luado(exarg_T *eap)
 	    luaV_emsg(L);
 	    break;
 	}
+	/* Catch the command switching to another buffer. */
+	if (curbuf != was_curbuf)
+	    break;
 	if (lua_isstring(L, -1)) /* update line? */
 	{
 #ifdef HAVE_SANDBOX
