@@ -1,4 +1,4 @@
-/* vi:set ts=8 sts=4 sw=4:
+/* vi:set ts=8 sts=4 sw=4 noet:
  *
  * VIM - Vi IMproved	by Bram Moolenaar
  * X command server by Flemming Madsen
@@ -167,7 +167,7 @@ struct ServerReply
 static garray_T serverReply = { 0, 0, 0, 0, 0 };
 enum ServerReplyOp { SROP_Find, SROP_Add, SROP_Delete };
 
-typedef int (*EndCond) __ARGS((void *));
+typedef int (*EndCond)(void *);
 
 struct x_cmdqueue
 {
@@ -186,21 +186,21 @@ static x_queue_T head = {NULL, 0, NULL, NULL};
  * Forward declarations for procedures defined later in this file:
  */
 
-static Window	LookupName __ARGS((Display *dpy, char_u *name, int delete, char_u **loose));
-static int	SendInit __ARGS((Display *dpy));
-static int	DoRegisterName __ARGS((Display *dpy, char_u *name));
-static void	DeleteAnyLingerer __ARGS((Display *dpy, Window w));
-static int	GetRegProp __ARGS((Display *dpy, char_u **regPropp, long_u *numItemsp, int domsg));
-static int	WaitForPend __ARGS((void *p));
-static int	WaitForReply __ARGS((void *p));
-static int	WindowValid __ARGS((Display *dpy, Window w));
-static void	ServerWait __ARGS((Display *dpy, Window w, EndCond endCond, void *endData, int localLoop, int seconds));
-static struct ServerReply *ServerReplyFind __ARGS((Window w, enum ServerReplyOp op));
-static int	AppendPropCarefully __ARGS((Display *display, Window window, Atom property, char_u *value, int length));
-static int	x_error_check __ARGS((Display *dpy, XErrorEvent *error_event));
-static int	IsSerialName __ARGS((char_u *name));
-static void	save_in_queue __ARGS((char_u *buf, long_u len));
-static void	server_parse_message __ARGS((Display *dpy, char_u *propInfo, long_u numItems));
+static Window	LookupName(Display *dpy, char_u *name, int delete, char_u **loose);
+static int	SendInit(Display *dpy);
+static int	DoRegisterName(Display *dpy, char_u *name);
+static void	DeleteAnyLingerer(Display *dpy, Window w);
+static int	GetRegProp(Display *dpy, char_u **regPropp, long_u *numItemsp, int domsg);
+static int	WaitForPend(void *p);
+static int	WaitForReply(void *p);
+static int	WindowValid(Display *dpy, Window w);
+static void	ServerWait(Display *dpy, Window w, EndCond endCond, void *endData, int localLoop, int seconds);
+static struct ServerReply *ServerReplyFind(Window w, enum ServerReplyOp op);
+static int	AppendPropCarefully(Display *display, Window window, Atom property, char_u *value, int length);
+static int	x_error_check(Display *dpy, XErrorEvent *error_event);
+static int	IsSerialName(char_u *name);
+static void	save_in_queue(char_u *buf, long_u len);
+static void	server_parse_message(Display *dpy, char_u *propInfo, long_u numItems);
 
 /* Private variables for the "server" functionality */
 static Atom	registryProperty = None;
@@ -214,9 +214,9 @@ static char_u	*empty_prop = (char_u *)"";	/* empty GetRegProp() result */
  * Returns FAIL or OK.
  */
     int
-serverRegisterName(dpy, name)
-    Display	*dpy;		/* display to register with */
-    char_u	*name;		/* the name that will be used as a base */
+serverRegisterName(
+    Display	*dpy,		/* display to register with */
+    char_u	*name)		/* the name that will be used as a base */
 {
     int		i;
     int		res;
@@ -231,7 +231,7 @@ serverRegisterName(dpy, name)
 	    if (res < -1 || i >= 1000)
 	    {
 		MSG_ATTR(_("Unable to register a command server name"),
-							      hl_attr(HLF_W));
+							      HL_ATTR(HLF_W));
 		return FAIL;
 	    }
 	    if (p == NULL)
@@ -252,9 +252,7 @@ serverRegisterName(dpy, name)
 }
 
     static int
-DoRegisterName(dpy, name)
-    Display	*dpy;
-    char_u	*name;
+DoRegisterName(Display *dpy, char_u *name)
 {
     Window	w;
     XErrorHandler old_handler;
@@ -332,9 +330,9 @@ DoRegisterName(dpy, name)
  * Change any registered window ID.
  */
     void
-serverChangeRegisteredWindow(dpy, newwin)
-    Display	*dpy;		/* Display to register with */
-    Window	newwin;		/* Re-register to this ID */
+serverChangeRegisteredWindow(
+    Display	*dpy,		/* Display to register with */
+    Window	newwin)		/* Re-register to this ID */
 {
     char_u	propInfo[MAX_NAME_LENGTH + 20];
 
@@ -368,15 +366,16 @@ serverChangeRegisteredWindow(dpy, newwin)
  * Returns 0 for OK, negative for an error.
  */
     int
-serverSendToVim(dpy, name, cmd,  result, server, asExpr, localLoop, silent)
-    Display	*dpy;			/* Where to send. */
-    char_u	*name;			/* Where to send. */
-    char_u	*cmd;			/* What to send. */
-    char_u	**result;		/* Result of eval'ed expression */
-    Window	*server;		/* Actual ID of receiving app */
-    Bool	asExpr;			/* Interpret as keystrokes or expr ? */
-    Bool	localLoop;		/* Throw away everything but result */
-    int		silent;			/* don't complain about no server */
+serverSendToVim(
+    Display	*dpy,			/* Where to send. */
+    char_u	*name,			/* Where to send. */
+    char_u	*cmd,			/* What to send. */
+    char_u	**result,		/* Result of eval'ed expression */
+    Window	*server,		/* Actual ID of receiving app */
+    Bool	asExpr,			/* Interpret as keystrokes or expr ? */
+    int		timeout,		/* seconds to wait or zero */
+    Bool	localLoop,		/* Throw away everything but result */
+    int		silent)			/* don't complain about no server */
 {
     Window	    w;
     char_u	    *property;
@@ -401,27 +400,7 @@ serverSendToVim(dpy, name, cmd,  result, server, asExpr, localLoop, silent)
 
     /* Execute locally if no display or target is ourselves */
     if (dpy == NULL || (serverName != NULL && STRICMP(name, serverName) == 0))
-    {
-	if (asExpr)
-	{
-	    char_u *ret;
-
-	    ret = eval_client_expr_to_string(cmd);
-	    if (result != NULL)
-	    {
-		if (ret == NULL)
-		    *result = vim_strsave((char_u *)_(e_invexprmsg));
-		else
-		    *result = ret;
-	    }
-	    else
-		vim_free(ret);
-	    return ret == NULL ? -1 : 0;
-	}
-	else
-	    server_to_input_buf(cmd);
-	return 0;
-    }
+	return sendToLocalVim(cmd, asExpr, result);
 
     /*
      * Bind the server name to a communication window.
@@ -441,6 +420,7 @@ serverSendToVim(dpy, name, cmd,  result, server, asExpr, localLoop, silent)
 	    {
 		LookupName(dpy, loosename ? loosename : name,
 			   /*DELETE=*/TRUE, NULL);
+		vim_free(loosename);
 		continue;
 	    }
 	}
@@ -507,7 +487,8 @@ serverSendToVim(dpy, name, cmd,  result, server, asExpr, localLoop, silent)
     pending.nextPtr = pendingCommands;
     pendingCommands = &pending;
 
-    ServerWait(dpy, w, WaitForPend, &pending, localLoop, 600);
+    ServerWait(dpy, w, WaitForPend, &pending, localLoop,
+						  timeout > 0 ? timeout : 600);
 
     /*
      * Unregister the information about the pending command
@@ -535,8 +516,7 @@ serverSendToVim(dpy, name, cmd,  result, server, asExpr, localLoop, silent)
 }
 
     static int
-WaitForPend(p)
-    void    *p;
+WaitForPend(void *p)
 {
     PendingCommand *pending = (PendingCommand *) p;
     return pending->result != NULL;
@@ -546,9 +526,7 @@ WaitForPend(p)
  * Return TRUE if window "w" exists and has a "Vim" property on it.
  */
     static int
-WindowValid(dpy, w)
-    Display     *dpy;
-    Window	w;
+WindowValid(Display *dpy, Window w)
 {
     XErrorHandler   old_handler;
     Atom	    *plist;
@@ -577,13 +555,13 @@ WindowValid(dpy, w)
  * Enter a loop processing X events & polling chars until we see a result
  */
     static void
-ServerWait(dpy, w, endCond, endData, localLoop, seconds)
-    Display	*dpy;
-    Window	w;
-    EndCond	endCond;
-    void	*endData;
-    int		localLoop;
-    int		seconds;
+ServerWait(
+    Display	*dpy,
+    Window	w,
+    EndCond	endCond,
+    void	*endData,
+    int		localLoop,
+    int		seconds)
 {
     time_t	    start;
     time_t	    now;
@@ -611,6 +589,7 @@ ServerWait(dpy, w, endCond, endData, localLoop, seconds)
     {
 	while (XCheckWindowEvent(dpy, commWindow, PropertyChangeMask, &event))
 	    serverEventProc(dpy, &event, 1);
+	server_parse_messages();
 
 	if (endCond(endData) != 0)
 	    break;
@@ -620,6 +599,10 @@ ServerWait(dpy, w, endCond, endData, localLoop, seconds)
 	if (seconds >= 0 && (now - start) >= seconds)
 	    break;
 
+#ifdef FEAT_TIMERS
+	check_due_timer();
+#endif
+
 	/* Just look out for the answer without calling back into Vim */
 	if (localLoop)
 	{
@@ -649,8 +632,7 @@ ServerWait(dpy, w, endCond, endData, localLoop, seconds)
  * Returns a newline separated list in allocated memory or NULL.
  */
     char_u *
-serverGetVimNames(dpy)
-    Display	*dpy;
+serverGetVimNames(Display *dpy)
 {
     char_u	*regProp;
     char_u	*entry;
@@ -704,9 +686,7 @@ serverGetVimNames(dpy)
  */
 
     static struct ServerReply *
-ServerReplyFind(w, op)
-    Window  w;
-    enum ServerReplyOp op;
+ServerReplyFind(Window w, enum ServerReplyOp op)
 {
     struct ServerReply *p;
     struct ServerReply e;
@@ -748,8 +728,7 @@ ServerReplyFind(w, op)
  * Issue an error if the id is invalid.
  */
     Window
-serverStrToWin(str)
-    char_u  *str;
+serverStrToWin(char_u *str)
 {
     unsigned  id = None;
 
@@ -765,9 +744,7 @@ serverStrToWin(str)
  * Return -1 if the window is invalid.
  */
     int
-serverSendReply(name, str)
-    char_u	*name;
-    char_u	*str;
+serverSendReply(char_u *name, char_u *str)
 {
     char_u	*property;
     int		length;
@@ -807,30 +784,33 @@ serverSendReply(name, str)
 }
 
     static int
-WaitForReply(p)
-    void    *p;
+WaitForReply(void *p)
 {
     Window  *w = (Window *) p;
+
     return ServerReplyFind(*w, SROP_Find) != NULL;
 }
 
 /*
  * Wait for replies from id (win)
+ * When "timeout" is non-zero wait up to this many seconds.
  * Return 0 and the malloc'ed string when a reply is available.
  * Return -1 if the window becomes invalid while waiting.
  */
     int
-serverReadReply(dpy, win, str, localLoop)
-    Display	*dpy;
-    Window	win;
-    char_u	**str;
-    int		localLoop;
+serverReadReply(
+    Display	*dpy,
+    Window	win,
+    char_u	**str,
+    int		localLoop,
+    int		timeout)
 {
     int		len;
     char_u	*s;
     struct	ServerReply *p;
 
-    ServerWait(dpy, win, WaitForReply, &win, localLoop, -1);
+    ServerWait(dpy, win, WaitForReply, &win, localLoop,
+						   timeout > 0 ? timeout : -1);
 
     if ((p = ServerReplyFind(win, SROP_Find)) != NULL && p->strings.ga_len > 0)
     {
@@ -858,10 +838,7 @@ serverReadReply(dpy, win, str, localLoop)
  * Return TRUE and a non-malloc'ed string if there is.  Else return FALSE.
  */
     int
-serverPeekReply(dpy, win, str)
-    Display *dpy;
-    Window win;
-    char_u **str;
+serverPeekReply(Display *dpy, Window win, char_u **str)
 {
     struct ServerReply *p;
 
@@ -882,8 +859,7 @@ serverPeekReply(dpy, win, str)
  * results.
  */
     static int
-SendInit(dpy)
-    Display *dpy;
+SendInit(Display *dpy)
 {
     XErrorHandler old_handler;
 
@@ -939,11 +915,11 @@ SendInit(dpy)
  *	removed from the registry property.
  */
     static Window
-LookupName(dpy, name, delete, loose)
-    Display	*dpy;	    /* Display whose registry to check. */
-    char_u	*name;	    /* Name of a server. */
-    int		delete;	    /* If non-zero, delete info about name. */
-    char_u	**loose;    /* Do another search matching -999 if not found
+LookupName(
+    Display	*dpy,	    /* Display whose registry to check. */
+    char_u	*name,	    /* Name of a server. */
+    int		delete,	    /* If non-zero, delete info about name. */
+    char_u	**loose)    /* Do another search matching -999 if not found
 			       Return result here if a match is found */
 {
     char_u	*regProp, *entry;
@@ -1033,9 +1009,9 @@ LookupName(dpy, name, delete, loose)
  * 3. The window will mistakenly be regarded valid because of own commWindow
  */
     static void
-DeleteAnyLingerer(dpy, win)
-    Display	*dpy;	/* Display whose registry to check. */
-    Window	win;	/* Window to remove */
+DeleteAnyLingerer(
+    Display	*dpy,	/* Display whose registry to check. */
+    Window	win)	/* Window to remove */
 {
     char_u	*regProp, *entry = NULL;
     char_u	*p;
@@ -1095,11 +1071,11 @@ DeleteAnyLingerer(dpy, win)
  * Return OK when successful.
  */
     static int
-GetRegProp(dpy, regPropp, numItemsp, domsg)
-    Display	*dpy;
-    char_u	**regPropp;
-    long_u	*numItemsp;
-    int		domsg;		/* When TRUE give error message. */
+GetRegProp(
+    Display	*dpy,
+    char_u	**regPropp,
+    long_u	*numItemsp,
+    int		domsg)		/* When TRUE give error message. */
 {
     int		result, actualFormat;
     long_u	bytesAfter;
@@ -1152,10 +1128,10 @@ GetRegProp(dpy, regPropp, numItemsp, domsg)
  * response.
  */
     void
-serverEventProc(dpy, eventPtr, immediate)
-    Display	*dpy;
-    XEvent	*eventPtr;	/* Information about event. */
-    int		immediate;	/* Run event immediately. Should mostly be 0. */
+serverEventProc(
+    Display	*dpy,
+    XEvent	*eventPtr,	/* Information about event. */
+    int		immediate)	/* Run event immediately. Should mostly be 0. */
 {
     char_u	*propInfo;
     int		result, actualFormat;
@@ -1197,9 +1173,7 @@ serverEventProc(dpy, eventPtr, immediate)
  * vim is idle.
  */
     static void
-save_in_queue(propInfo, len)
-    char_u	*propInfo;
-    long_u	len;
+save_in_queue(char_u *propInfo, long_u len)
 {
     x_queue_T *node;
 
@@ -1226,7 +1200,7 @@ save_in_queue(propInfo, len)
  * Parses queued clientserver messages.
  */
     void
-server_parse_messages()
+server_parse_messages(void)
 {
     x_queue_T	*node;
 
@@ -1247,7 +1221,7 @@ server_parse_messages()
  * int the queue.
  */
     int
-server_waiting()
+server_waiting(void)
 {
     return head.next != NULL && head.next != &head;
 }
@@ -1258,10 +1232,10 @@ server_waiting()
  * "propInfo" will be freed.
  */
     static void
-server_parse_message(dpy, propInfo, numItems)
-    Display	*dpy;
-    char_u	*propInfo; /* A string containing 0 or more X commands */
-    long_u	numItems;  /* The size of propInfo in bytes. */
+server_parse_message(
+    Display	*dpy,
+    char_u	*propInfo, /* A string containing 0 or more X commands */
+    long_u	numItems)  /* The size of propInfo in bytes. */
 {
     char_u	*p;
     int		code;
@@ -1466,8 +1440,8 @@ server_parse_message(dpy, propInfo, numItems)
 	    char_u	*enc;
 
 	    /*
-	     * This is a (n)otification.  Sent with serverreply_send in VimL.
-	     * Execute any autocommand and save it for later retrieval
+	     * This is a (n)otification.  Sent with serverreply_send in Vim
+	     * script.  Execute any autocommand and save it for later retrieval
 	     */
 	    p += 2;
 	    gotWindow = 0;
@@ -1539,12 +1513,12 @@ server_parse_message(dpy, propInfo, numItems)
  * Return: 0 for OK, -1 for error
  */
     static int
-AppendPropCarefully(dpy, window, property, value, length)
-    Display	*dpy;		/* Display on which to operate. */
-    Window	window;		/* Window whose property is to be modified. */
-    Atom	property;	/* Name of property. */
-    char_u	*value;		/* Characters  to append to property. */
-    int		length;		/* How much to append */
+AppendPropCarefully(
+    Display	*dpy,		/* Display on which to operate. */
+    Window	window,		/* Window whose property is to be modified. */
+    Atom	property,	/* Name of property. */
+    char_u	*value,		/* Characters  to append to property. */
+    int		length)		/* How much to append */
 {
     XErrorHandler old_handler;
 
@@ -1562,9 +1536,7 @@ AppendPropCarefully(dpy, window, property, value, length)
  * Another X Error handler, just used to check for errors.
  */
     static int
-x_error_check(dpy, error_event)
-    Display	*dpy UNUSED;
-    XErrorEvent	*error_event UNUSED;
+x_error_check(Display *dpy UNUSED, XErrorEvent *error_event UNUSED)
 {
     got_x_error = TRUE;
     return 0;
@@ -1575,8 +1547,7 @@ x_error_check(dpy, error_event)
  * Actually just checks if the name ends in a digit.
  */
     static int
-IsSerialName(str)
-    char_u	*str;
+IsSerialName(char_u *str)
 {
     int len = STRLEN(str);
 
