@@ -1,4 +1,4 @@
-/* vi:set ts=8 sw=8:
+/* vi:set ts=8 sw=8 noet:
  *
  * VIM - Vi IMproved	by Bram Moolenaar
  *			Visual Workshop integration by Gordon Prieur
@@ -620,8 +620,8 @@ adjust_sign_name(char *filename)
    if you want to do things differently if the editor is started standalone
    or in WorkShop mode. For example, in standalone mode you may not want to
    add a footer/message area or a sign gutter. */
-int
-workshop_invoked()
+	int
+workshop_invoked(void)
 {
 	static int result = -1;
 	if (result == -1) {
@@ -735,7 +735,7 @@ void	workshop_connect(XtAppContext context)
 	dummy = write(sd, buf, strlen(buf));
 }
 
-void	workshop_disconnect()
+void	workshop_disconnect(void)
 {
 	/* Probably need to send some message here */
 
@@ -805,7 +805,7 @@ widgetIsIconified(
 	if (XtWindow(w) != 0) {			/* only check if window exists! */
 		XGetWindowProperty(XtDisplay(w), XtWindow(w), wm_state, 0L, 2L,
 		    False, AnyPropertyType, &act_type, &act_fmt, &nitems_ret,
-		    &bytes_after, (u_char **) &property);
+		    &bytes_after, (char_u **) &property);
 		if (nitems_ret == 2 && property[0] == IconicState) {
 			return True;
 		}
@@ -1077,7 +1077,7 @@ void workshop_perform_verb(char *verb, void *clientData)
 }
 
 /* Send a message to eserve */
-#if defined(NOHANDS_SUPPORT_FUNCTIONS) || defined(FEAT_BEVAL)
+#if defined(NOHANDS_SUPPORT_FUNCTIONS) || defined(FEAT_BEVAL_GUI)
 void workshop_send_message(char *buf)
 {
 	dummy = write(sd, buf, strlen(buf));
